"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _mongodb = require("mongodb");

var _helpers = require("../libs/helpers");

var _default = Repository => ({
  findById: {
    loader: async ids => {
      const items = (0, _helpers.fromMongoArray)((await Repository.findMany({
        query: {
          _id: {
            $in: ids.map(_mongodb.ObjectID)
          }
        }
      })));
      return ids.map(id => items.find(item => item.id.toString() === id.toString()) || null);
    },
    options: {
      cacheKeyFn: key => key.toString()
    }
  },
  findOne: {
    loader: queries => Promise.all(queries.map(query => Repository.findOne(query).then(item => item ? (0, _helpers.fromMongo)(item) : null))),
    options: {
      cacheKeyFn: key => JSON.stringify(key)
    }
  },
  findMany: {
    loader: queries => Promise.all(queries.map((query = {}) => Repository.findMany(query).then(_helpers.fromMongoArray))),
    options: {
      cacheKeyFn: key => JSON.stringify(key)
    }
  },
  aggregate: {
    loader: pipelines => Promise.all(pipelines.map(pipeline => Repository.aggregate(pipeline))),
    options: {
      cacheKeyFn: key => JSON.stringify(key)
    }
  },
  count: {
    loader: queries => Promise.all(queries.map((query = {}) => Repository.count(query))),
    options: {
      cacheKeyFn: key => JSON.stringify(key)
    }
  }
});

exports.default = _default;