"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.deleteOne = exports.updateOne = exports.createMany = exports.createOne = exports.count = exports.findMany = exports.findOne = exports.findById = exports.buildConfig = void 0;

var _easevalidation = require("easevalidation");

var _invariant = _interopRequireDefault(require("invariant"));

var _identity = _interopRequireDefault(require("lodash/identity"));

var _omit = _interopRequireDefault(require("lodash/omit"));

var _withValidator = _interopRequireDefault(require("oors-graphql/build/decorators/withValidator"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const buildConfig = config => {
  if (typeof config === 'string') {
    return buildConfig({
      repositoryName: config
    });
  }

  const {
    wrapPipeline,
    nodeVisitors,
    getInitialPipeline,
    repositoryName
  } = _objectSpread({
    wrapPipeline: () => _identity.default,
    getInitialPipeline: (_, args, ctx, info, pipeline) => pipeline,
    nodeVisitors: []
  }, config);

  if (repositoryName) {
    if (!config.getRepository) {
      Object.assign(config, {
        getRepository: ({
          getRepository
        }) => getRepository(repositoryName)
      });
    }

    if (!config.getLoaders) {
      Object.assign(config, {
        getLoaders: ({
          modules,
          loaders
        }) => loaders[modules.get('oors.rad').getLoadersName(repositoryName)]
      });
    }
  }

  (0, _invariant.default)(typeof config.getRepository === 'string' || typeof config.getRepository === 'function', `Invalid required getRepository parameter (needs to be a repository name or a function that 
      will receive a resolver context as argument)`);
  (0, _invariant.default)(typeof config.getLoaders === 'function', `Invalid required getLoaders parameter (needs to be a function that will receive a resolver 
      context as argument and returns DataLoader instances)`);
  const getRepository = typeof config.getRepository === 'string' ? ctx => ctx.getRepository(config.getRepository) : config.getRepository;

  const createPipeline = (_, args, ctx, info) => {
    const repository = getRepository(ctx);
    const pipeline = args.pipeline || getInitialPipeline(_, args, ctx, info, repository.createPipeline());
    return wrapPipeline(_, args, ctx, info)(ctx.gqlQueryParser.toPipeline(args, {
      repository,
      pipeline,
      nodeVisitors
    }));
  };

  return _objectSpread({
    createPipeline,
    getRepository,
    canDelete: () => true,
    canUpdate: () => true
  }, config);
};

exports.buildConfig = buildConfig;

const findById = config => {
  const {
    getLoaders
  } = buildConfig(config);
  return (_, {
    id
  }, ctx) => getLoaders(ctx).findById.load(id);
};

exports.findById = findById;

const findOne = config => {
  const {
    createPipeline
  } = buildConfig(config);
  return (0, _withValidator.default)(_easevalidation.validators.isSchema({
    where: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isRequired(), _easevalidation.validators.isObject()]
  }))(async (_, args, ctx, info) => ctx.fromMongo((await createPipeline(_, args, ctx, info).one().toResult())));
};

exports.findOne = findOne;

const findMany = config => {
  const {
    getLoaders,
    createPipeline
  } = buildConfig(config);
  return async (_, args, ctx, info) => {
    const pivot = args.before || args.after;

    if (pivot) {
      Object.assign(args, {
        pivot: await getLoaders(ctx).findById.load(pivot)
      });
    }

    return ctx.fromMongoArray((await createPipeline(_, args, ctx, info).toArray()));
  };
};

exports.findMany = findMany;

const count = config => {
  const {
    createPipeline
  } = buildConfig(config);
  return async (_, args, ctx, info) => createPipeline(_, args, ctx, info).count().toResult();
};

exports.count = count;

const createOne = config => {
  const {
    getRepository
  } = buildConfig(config);
  return async (_, {
    input
  }, ctx) => ctx.fromMongo((await getRepository(ctx).createOne(input)));
};

exports.createOne = createOne;

const createMany = config => {
  const {
    getRepository
  } = buildConfig(config);
  return async (_, args, ctx) => (await getRepository(ctx).createMany(args.input)).map(ctx.fromMongo);
};

exports.createMany = createMany;

const updateOne = config => {
  const {
    getRepository,
    createPipeline,
    canUpdate
  } = buildConfig(config);
  return async (_, args, ctx, info) => {
    const {
      input
    } = args;
    let {
      item
    } = args;
    const Repository = getRepository(ctx);

    if (item === undefined) {
      item = await createPipeline(_, args, ctx, info).one().toResult();
    }

    if (!item) {
      throw new Error('Unable to find item!');
    }

    if (!canUpdate(_, args, ctx, info, item)) {
      throw new Error('Not Allowed!');
    }

    await Repository.validate((0, _omit.default)(_objectSpread({}, item, {}, input), ['_id']));
    return ctx.fromMongo((await Repository.updateOne({
      query: {
        _id: item._id
      },
      update: {
        $set: input
      }
    })));
  };
};

exports.updateOne = updateOne;

const deleteOne = config => {
  const {
    getRepository,
    createPipeline,
    canDelete
  } = buildConfig(config);
  return async (_, args, ctx, info) => {
    const Repository = getRepository(ctx);
    let {
      item
    } = args;

    if (item === undefined) {
      item = await createPipeline(_, args, ctx, info).one().toResult();
    }

    if (!item) {
      throw new Error('Unable to find item!');
    }

    if (!canDelete(_, args, ctx, info, item)) {
      throw new Error('Not Allowed!');
    }

    return ctx.fromMongo((await Repository.deleteOne({
      query: {
        _id: item._id
      }
    })));
  };
};

exports.deleteOne = deleteOne;

var _default = config => ({
  findById: findById(config),
  findOne: findOne(config),
  findMany: findMany(config),
  count: count(config),
  createOne: createOne(config),
  createMany: createMany(config),
  updateOne: updateOne(config),
  deleteOne: deleteOne(config)
});

exports.default = _default;