"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "Repository", {
  enumerable: true,
  get: function () {
    return _Repository.default;
  }
});
Object.defineProperty(exports, "Migration", {
  enumerable: true,
  get: function () {
    return _Migration2.default;
  }
});
exports.decorators = exports.helpers = exports.default = void 0;

var _easevalidation = require("easevalidation");

var _invariant = _interopRequireDefault(require("invariant"));

var _merge = _interopRequireDefault(require("lodash/merge"));

var _mongodb = require("mongodb");

var _oors = require("oors");

var _Repository = _interopRequireDefault(require("./libs/Repository"));

var helpers = _interopRequireWildcard(require("./libs/helpers"));

exports.helpers = helpers;

var decorators = _interopRequireWildcard(require("./decorators"));

exports.decorators = decorators;

var _Migration = _interopRequireDefault(require("./repositories/Migration"));

var _Seeder = _interopRequireDefault(require("./libs/Seeder"));

var _withLogger = _interopRequireDefault(require("./decorators/withLogger"));

var _withTimestamps = _interopRequireDefault(require("./decorators/withTimestamps"));

var _Migration2 = _interopRequireDefault(require("./libs/Migration"));

var _GQLQueryParser = _interopRequireDefault(require("./graphql/GQLQueryParser"));

var _Migrator = _interopRequireDefault(require("./libs/Migrator"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class MongoDB extends _oors.Module {
  constructor(..._args) {
    super(..._args);
    this.name = 'oors.mongodb';
    this.connections = {};
    this.relations = {};
    this.repositories = {};
    this.hooks = {
      'oors.graphql.buildContext': ({
        context
      }) => {
        const {
          fromMongo,
          fromMongoCursor,
          fromMongoArray,
          toMongo
        } = helpers;
        Object.assign(context, {
          fromMongo,
          fromMongoCursor,
          fromMongoArray,
          toMongo,
          getRepository: this.getRepository,
          toObjectId: this.toObjectId,
          gqlQueryParser: this.gqlQueryParser
        });
      }
    };

    this.teardown = () => Promise.all(Object.keys(this.connections).map(connectionName => this.closeConnection(connectionName)));

    this.loadFromModules = async () => {
      await Promise.all([this.runHook('loadRepositories', this.loadRepositoriesFromModule, {
        createRepository: this.createRepository,
        bindRepositories: this.bindRepository,
        bindRepository: this.bindRepository
      }), this.getConfig('migration.isEnabled') ? this.runHook('loadMigrations', this.loadMigrationsFromModule, {
        migrator: this.migrator
      }) : Promise.resolve()]);
      this.configureRepositories();
    };

    this.getModuleConfig = module => (0, _merge.default)({}, this.getConfig('moduleDefaultConfig'), module.getConfig(this.name));

    this.loadRepositoriesFromModule = async module => {
      const config = this.getModuleConfig(module);

      if (!config.repositories.autoload) {
        return;
      }

      const {
        glob
      } = this.deps['oors.autoloader'].wrap(module);
      const files = await glob(`${config.repositories.dir}/*.js`, {
        nodir: true
      });
      files.forEach(file => {
        const ModuleRepository = require(file).default; // eslint-disable-line global-require, import/no-dynamic-require


        const repository = new ModuleRepository();

        if (config.repositories.collectionPrefix) {
          repository.collectionName = `${config.repositories.collectionPrefix}${repository.collectionName}`;
        }

        repository.module = module;
        const name = `${config.repositories.prefix || module.name}.${repository.name || repository.constructor.name}`;
        this.addRepository(name, repository);
        module.export(`repositories.${repository.name || repository.constructor.name}`, repository);
      });
    };

    this.loadMigrationsFromModule = async module => {
      const config = this.getModuleConfig(module);

      if (!config.migrations.autoload) {
        return;
      }

      const {
        glob
      } = this.deps['oors.autoloader'].wrap(module);
      const files = await glob(`${config.migrations.dir}/*.js`, {
        nodir: true
      });
      this.migrator.files.push(...files);
    };

    this.createConnection = async ({
      name,
      url,
      options
    }) => {
      this.connections[name] = await _mongodb.MongoClient.connect(url, _objectSpread({}, options, {
        useNewUrlParser: true
      }));
      return this.connections[name];
    };

    this.getConnectionDb = (name = this.defaultConnectionName) => {
      const connection = this.getConnection(name);
      const {
        database,
        url
      } = this.getConfig('connections').find(({
        name: _name
      }) => _name === name);
      return connection.db(database || url.substr(url.lastIndexOf('/') + 1));
    };

    this.getConnection = name => {
      if (!name) {
        return this.connections[this.defaultConnectionName];
      }

      if (!this.connections[name]) {
        throw new Error(`Unknown connection name - "${name}"!`);
      }

      return this.connections[name];
    };

    this.closeConnection = name => this.getConnection(name).close();

    this.toObjectId = value => new _mongodb.ObjectID(value);

    this.transaction = async (cb, options = {}, connectionName) => {
      const db = this.getConnectionDb(connectionName);

      if (!this.getConfig('transaction.isEnabled')) {
        return cb(db);
      }

      const session = db.startSession();
      session.startTransaction(options);

      try {
        const result = await cb(db);
        await session.commitTransaction();
        session.endSession();
        return result;
      } catch (error) {
        await session.abortTransaction();
        session.endSession();
        throw error;
      }
    };

    this.backup = connectionName => {// https://github.com/theycallmeswift/node-mongodb-s3-backup
      // https://dzone.com/articles/auto-backup-mongodb-database-with-nodejs-on-server-1
    };

    this.extractCollectionName = relationNode => relationNode.collectionName || relationNode.repositoryName && this.getRepository(relationNode.repositoryName).collectionName || relationNode.repository && relationNode.repository.collectionName;

    this.addRelation = (_ref) => {
      let {
        type,
        inversedType
      } = _ref,
          args = _objectWithoutProperties(_ref, ["type", "inversedType"]);

      const from = _objectSpread({}, args.from, {
        collectionName: this.extractCollectionName(args.from)
      });

      const to = _objectSpread({}, args.to, {
        collectionName: this.extractCollectionName(args.to)
      });

      if (!this.relations[from.collectionName]) {
        this.relations[from.collectionName] = {};
      }

      this.relations[from.collectionName][from.name] = {
        collectionName: to.collectionName,
        localField: from.field,
        foreignField: to.field,
        type
      };

      if (inversedType && to.name) {
        this.addRelation({
          from: to,
          to: from,
          type: inversedType
        });
      }
    };

    this.relationToLookup = (collectionName, name) => ({
      from: this.relations[collectionName][name].collectionName,
      localField: this.relations[collectionName][name].localField,
      foreignField: this.relations[collectionName][name].foreignField,
      as: name
    });

    this.createRepository = (_ref2) => {
      let {
        methods = {},
        connectionName
      } = _ref2,
          options = _objectWithoutProperties(_ref2, ["methods", "connectionName"]);

      const repository = new _Repository.default(options);
      Object.keys(methods).forEach(methodName => {
        repository[methodName] = methods[methodName].bind(repository);
      });
      this.bindRepository(repository, connectionName);
      return repository;
    };

    this.bindRepository = (repository, connectionName) => {
      if (Array.isArray(repository)) {
        return repository.map(repo => this.bind(repo, connectionName));
      }

      (0, _invariant.default)(repository.collectionName, `Missing repository collection name - ${repository.constructor.name}!`);
      Object.assign(repository, {
        collection: !repository.hasCollection() ? this.getConnectionDb(connectionName).collection(repository.collectionName) : repository.collection,
        getRepository: this.getRepository,
        relationToLookup: (name, options = {}) => _objectSpread({}, this.relationToLookup(repository.collectionName, name), {}, options),
        getRelation: name => this.relations[repository.collectionName][name],
        hasRelation: name => this.relations[repository.collectionName][name] !== undefined
      });
      return repository;
    };

    this.addRepository = (key, repository, options = {}) => {
      const payload = {
        key,
        repository,
        options
      };
      this.emit('repository', payload);
      this.repositories[payload.key] = this.bindRepository(payload.repository, options.connectionName);
      return this.repositories[payload.key];
    };

    this.addRepositories = repositories => Object.keys(repositories).reduce((acc, repositoryName) => _objectSpread({}, acc, {
      [repositoryName]: this.addRepository(repositoryName, repositories[repositoryName])
    }), {});

    this.getRepository = key => {
      if (!this.repositories[key]) {
        throw new Error(`Unable to find "${key}" repository!`);
      }

      return this.repositories[key];
    };

    this.configureRepositories = (repositories = this.repositories) => {
      Object.keys(repositories).forEach(key => {
        const repository = repositories[key];
        repository.configure({
          getRepository: this.getRepository
        });
        Object.keys(repository.relations).forEach(relationName => {
          const _repository$relations = repository.relations[relationName],
                {
            localField,
            foreignField,
            type
          } = _repository$relations,
                restOptions = _objectWithoutProperties(_repository$relations, ["localField", "foreignField", "type"]);

          this.addRelation({
            from: {
              repository,
              field: localField,
              name: relationName
            },
            to: _objectSpread({}, restOptions, {
              field: foreignField
            }),
            type
          });
        });
      });
    };
  }

  initialize({
    connections,
    defaultConnection,
    logQueries,
    addTimestamps
  }) {
    this.defaultConnectionName = defaultConnection || connections[0].name;
    const names = connections.map(({
      name
    }) => name);

    if (!names.includes(this.defaultConnectionName)) {
      throw new Error(`Default connection name - "(${this.defaultConnectionName})" - can't be found through the list of available connections (${names})`);
    }

    this.onModule(this.name, 'repository', ({
      repository
    }) => {
      if (logQueries) {
        (0, _withLogger.default)()(repository);
      }

      if (addTimestamps) {
        (0, _withTimestamps.default)()(repository);
      }
    });
    this.addHook('oors.health', 'scan', async collector => {
      const databases = {};
      await Promise.all(Object.keys(this.connections).map(async name => {
        databases[name] = await this.connections[name].isConnected();
      }));
      Object.assign(collector, {
        'oors.mongodb': {
          databases
        }
      });
    });
  }

  async setup({
    connections
  }) {
    await this.loadDependencies(['oors.autoloader']);
    await Promise.all(connections.map(this.createConnection));

    if (this.getConfig('migration.isEnabled')) {
      this.createMigrator();
    }

    await this.loadFromModules();

    if (this.getConfig('seeding.isEnabled')) {
      await this.setupSeeding();
    }

    this.gqlQueryParser = new _GQLQueryParser.default(this);
    this.onModule('oors.graphql', 'healthCheck', async () => {
      await Promise.all(Object.keys(this.connections).map(async name => {
        const isConnected = await this.connections[name].isConnected();

        if (!isConnected) {
          throw new Error(`Connection closed - "${name}"!`);
        }
      }));
    });
    this.exportProperties(['createConnection', 'closeConnection', 'getConnection', 'getConnectionDb', 'toObjectId', 'gqlQueryParser', 'transaction', 'backup', 'repositories', 'createRepository', 'getRepository', 'addRepository', 'bindRepository', 'relations', 'addRelation', 'relationToLookup']);
    this.export({
      configureRelations: configure => configure({
        add: this.addRelation,
        relations: this.relations,
        RELATION_TYPE: this.constructor.RELATION_TYPE,
        getRepository: this.getRepository
      })
    });
  }

  createMigrator() {
    const migrationRepository = this.addRepository('Migration', new _Migration.default());
    this.migrator = new _Migrator.default({
      context: {
        modules: this.manager,
        db: this.getConnectionDb()
      },
      MigrationRepository: migrationRepository,
      transaction: this.transaction,
      backup: this.backup,
      getRepository: this.getRepository,
      silent: this.getConfig('migration.isSilent')
    });
    this.export({
      migrator: this.migrator,
      migrate: this.migrator.run
    });
  }

  async setupSeeding() {
    const seeder = new _Seeder.default();
    const seeds = {};
    await Promise.all([this.runHook('configureSeeder', () => {}, {
      seeder,
      getRepository: this.getRepository
    }), this.runHook('loadSeedData', () => {}, {
      seeds
    })]);

    if (Object.keys(seeds).length) {
      await this.seed(seeds);
    }

    this.export({
      seeder,
      seed: seeder.load
    });
  }

}

exports.default = MongoDB;
MongoDB.validateConfig = (0, _easevalidation.validate)(_easevalidation.validators.isSchema({
  connections: [_easevalidation.validators.isRequired(), _easevalidation.validators.isArray(_easevalidation.validators.isSchema({
    name: [_easevalidation.validators.isRequired(), _easevalidation.validators.isString()],
    database: _easevalidation.validators.isAny(_easevalidation.validators.isString(), _easevalidation.validators.isUndefined()),
    url: [_easevalidation.validators.isRequired(), _easevalidation.validators.isString()],
    options: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isObject()]
  })), _easevalidation.validators.isLength({
    min: 1
  })],
  defaultConnection: _easevalidation.validators.isAny(_easevalidation.validators.isString(), _easevalidation.validators.isUndefined()),
  migration: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    isEnabled: [_easevalidation.validators.isDefault(false), _easevalidation.validators.isBoolean()],
    isSilent: [_easevalidation.validators.isDefault(false), _easevalidation.validators.isBoolean()]
  })],
  logQueries: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
  addTimestamps: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
  seeding: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    isEnabled: [_easevalidation.validators.isDefault(false), _easevalidation.validators.isBoolean()]
  })],
  transaction: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    isEnabled: [_easevalidation.validators.isDefault(false), _easevalidation.validators.isBoolean()]
  })],
  moduleDefaultConfig: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    repositories: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
      autoload: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
      dir: [_easevalidation.validators.isDefault('repositories'), _easevalidation.validators.isString()],
      prefix: [_easevalidation.validators.isDefault(''), _easevalidation.validators.isString()],
      collectionPrefix: [_easevalidation.validators.isDefault(''), _easevalidation.validators.isString()]
    })],
    migrations: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
      autoload: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
      dir: [_easevalidation.validators.isDefault('migrations'), _easevalidation.validators.isString()]
    })]
  })]
}));
MongoDB.RELATION_TYPE = {
  ONE: 'one',
  MANY: 'many'
};