"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _last = _interopRequireDefault(require("lodash/last"));

var _isEqual = _interopRequireDefault(require("lodash/isEqual"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class AggregationPipeline {
  constructor(repository, startPipeline = [], _options = {}) {
    this.match = query => typeof query !== 'object' || !Object.keys(query).length ? this : this.push({
      $match: query
    });

    this.project = fields => this.push({
      $project: Array.isArray(fields) ? fields.reduce((acc, field) => _objectSpread({}, acc, {}, typeof field === 'string' ? {
        [field]: 1
      } : field), {}) : fields
    });

    this.lookup = (relation, _ref = {}) => {
      let {
        as = relation,
        match = false,
        projectOne = true,
        ignoreIfExists = false
      } = _ref,
          rest = _objectWithoutProperties(_ref, ["as", "match", "projectOne", "ignoreIfExists"]);

      if (typeof relation === 'object') {
        return this.push({
          $lookup: relation
        });
      }

      if (!this.repository.hasRelation(relation)) {
        throw new Error(`Missing "${relation}" relation in the "${this.repository.constructor.name}" repository!`);
      }

      const {
        type
      } = this.repository.getRelation(relation);

      if (ignoreIfExists && this.hasLookup(relation, _objectSpread({
        as
      }, rest))) {
        return this;
      }

      this.push({
        $lookup: this.repository.relationToLookup(relation, _objectSpread({
          as
        }, rest))
      });

      if (match) {
        this.match({
          [as]: {
            $elemMatch: match
          }
        });
      }

      if (type === 'one' && projectOne) {
        this.addFields({
          [as]: {
            $arrayElemAt: [`$${as}`, 0]
          }
        });
      }

      return this;
    };

    this.hasLookup = (relation, options = {}) => !!this.stack.find(stage => stage.$lookup && (0, _isEqual.default)(stage.$lookup, this.repository.relationToLookup(relation, options)));

    this.sort = fields => this.push({
      $sort: Object.keys(fields).reduce((acc, field) => _objectSpread({}, acc, {
        [field]: fields[field] === 1 || typeof fields[field] === 'string' && fields[field].toLowerCase() === 'asc' ? 1 : -1
      }), {})
    });

    this.slice = (start, end) => this.push(this.skip(start), this.limit(end - start));

    this.count = (outputField = 'count') => this.push({
      $count: outputField
    });

    this.one = () => this.limit(1);

    this.clone = pipeline => new this.constructor(this.repository, pipeline || [...this.stack], _objectSpread({}, this.options));

    this.toCursor = (options = {}) => this.repository.aggregate({
      pipeline: this.stack,
      options: _objectSpread({}, this.options, {}, options)
    });

    this.toArray = async options => (await this.toCursor(options)).toArray();

    this.toResult = async options => {
      const lastStage = (0, _last.default)(this.stack);
      const result = await this.toArray(options);

      if (!lastStage) {
        return result;
      }

      if (lastStage.$count) {
        return result.length > 0 ? result[0][lastStage.$count] : 0;
      }

      if (lastStage.$limit && lastStage.$limit === 1) {
        return result.length > 0 ? result[0] : null;
      }

      return result;
    };

    this.watch = (options = {}) => this.repository.collection.watch(this.stack, options);

    this.watchChanges = (onChange, ...args) => {
      const changeStream = this.watch(...args);
      changeStream.on('change', onChange);
      return changeStream;
    };

    this.on = (operation, cb, options = {}) => {
      const operationType = operation === 'create' ? 'insert' : operation;
      const changeStream = this.match({
        operationType
      }).watchChanges(change => cb(change.fullDocument, change), _objectSpread({
        fullDocument: 'updateLookup'
      }, options));
      return () => changeStream.close();
    };

    this.repository = repository;
    this.stack = [];
    this.options = _options;
    this.merge(startPipeline); // check if using Proxy is a better alternative

    this.constructor.stages.forEach(stage => {
      if (!this[stage]) {
        this[stage] = value => this.push({
          [`$${stage}`]: value
        });
      }
    });
  }

  setOptions(options) {
    if (typeof options === 'function') {
      this.options = options(this.options);
    } else {
      Object.assign(this.options, options);
    }

    return this;
  }

  unshift(...operations) {
    operations.reverse().forEach(operation => {
      this.stack.unshift(...(Array.isArray(operation) ? operation : [operation]));
    });
    return this;
  }

  push(...operations) {
    operations.forEach(operation => {
      this.stack.push(...(Array.isArray(operation) ? operation : [operation]));
    });
    return this;
  }

  merge(pipeline) {
    return this.push(Array.isArray(pipeline) ? pipeline : pipeline.stack);
  }

}

AggregationPipeline.stages = ['addFields', 'bucket', 'bucketAuto', 'collStats', 'count', 'currentOp', 'facet', 'geoNear', 'graphLookup', 'group', 'indexStats', 'limit', 'listLocalSessions', 'listSessions', 'lookup', 'match', 'out', 'project', 'redact', 'replaceRoot', 'sample', 'skip', 'sort', 'sortByCount', 'unwind'];
var _default = AggregationPipeline;
exports.default = _default;