"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _helpers = require("./helpers");

class Migrator {
  constructor({
    files = [],
    context,
    MigrationRepository,
    transaction,
    backup,
    getRepository,
    silent = false
  }) {
    this.print = message => {
      if (this.silent) {
        return;
      } // @TODO: print timestamp


      console.log(message);
    };

    this.run = async () => {
      await this.backup();

      if (!this.files.length) {
        throw new Error(`Missing migration files!`);
      }

      const lastDbMigration = await this.MigrationRepository.findOne({
        query: {
          error: null
        },
        options: {
          sort: [['timestamp', '-1']]
        }
      });
      const lastDbMigrationTimestamp = parseInt(lastDbMigration ? lastDbMigration.timestamp : 0, 10);
      return this.files.filter(file => parseInt((0, _helpers.getTimestampFromMigrationFile)(file), 10) > lastDbMigrationTimestamp).sort((f1, f2) => parseInt((0, _helpers.getTimestampFromMigrationFile)(f1), 10) - parseInt((0, _helpers.getTimestampFromMigrationFile)(f2), 10)).reduce((promise, file) => promise.then(() => this.runSingleMigration(file)), Promise.resolve());
    };

    this.runSingleMigration = async file => {
      const fileName = file.substr(file.lastIndexOf('/') + 1);
      const timestamp = fileName.substr(0, fileName.indexOf('_'));

      const MigrationClass = require(file).default; // eslint-disable-line import/no-dynamic-require, global-require


      const migration = new MigrationClass(this.context);

      try {
        await this.transaction(async () => {
          const startTime = Date.now();
          const result = await migration.up();
          const duration = Date.now() - startTime;
          await this.MigrationRepository.createOne({
            timestamp,
            name: migration.name,
            result,
            duration
          });
          this.print(`Completed migration - ${fileName}`);
        });
      } catch (error) {
        this.MigrationRepository.createOne({
          timestamp,
          name: migration.name,
          error
        });
        this.print(`An error ocurred when running the "${fileName}" migration!`);
        throw error;
      }
    };

    this.files = files;
    this.context = context;
    this.MigrationRepository = MigrationRepository;
    this.transaction = transaction;

    this.backup = async () => {
      this.print('Starting database backup.');
      await backup();
      this.print('Database backup completed.');
    };

    this.getRepository = getRepository;
    this.silent = silent;
  }

}

var _default = Migrator;
exports.default = _default;