"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _easevalidation = require("easevalidation");

var _get = _interopRequireDefault(require("lodash/get"));

var _Store = _interopRequireDefault(require("./Store"));

var _AggregationPipeline = _interopRequireDefault(require("./AggregationPipeline"));

var _validators = require("./validators");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Repository extends _Store.default {
  static getCollectionName() {
    return this.collectionName || this.name.replace(/Repository$/, '');
  }

  static validateRelations(relations) {
    return (0, _easevalidation.validate)(_easevalidation.validators.isObject({
      validator: _validators.isRelation
    }))(relations);
  }

  constructor({
    collection,
    collectionName,
    schema,
    relations
  } = {}) {
    super(collection);

    this.runBulkOperation = async (callback, {
      ordered = false
    } = {}) => {
      const bulk = this.collection[`initialize${ordered ? 'Ordered' : 'Unordered'}BulkOp`]();
      await callback(bulk);
      const hasOperations = (0, _get.default)(bulk, 's.currentBatch.operations', []).length > 0;
      return hasOperations ? bulk.execute() : false;
    };

    this.createPipeline = (...args) => new _AggregationPipeline.default(this, ...args);

    this.collectionName = collection && collection.collectionName || collectionName || this.collectionName || this.constructor.getCollectionName();
    this.relations = this.constructor.validateRelations(relations || this.relations || this.constructor.relations || {});
    this.validators = [];
    this.schema = schema || this.schema || this.constructor.schema;

    if (typeof this.schema === 'object') {
      this.validators.push(_easevalidation.validators.isSchema(this.schema));
    } else if (typeof this.schema === 'function') {
      this.validators.push(this.schema);
    }
  } // eslint-disable-next-line


  configure({
    getRepository
  }) {} // eslint-disable-next-line class-methods-use-this


  getRepository() {
    throw new Error('Not available! You need to bind the repository first.');
  }

  isValid(data) {
    if (!this.validators.length) {
      return true;
    }

    return (0, _easevalidation.test)(this.validators)(data);
  }

  validate(data) {
    if (!this.validators.length) {
      return data;
    }

    return (0, _easevalidation.validate)(this.validators)(data);
  }

  async parse(data) {
    return this.validate(data);
  }

}

var _default = Repository;
exports.default = _default;