"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _omit = _interopRequireDefault(require("lodash/omit"));

var _get = _interopRequireDefault(require("lodash/get"));

var _has = _interopRequireDefault(require("lodash/has"));

var _set = _interopRequireDefault(require("lodash/set"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class Seeder {
  static async resolveMap(map, fn) {
    const keys = Object.keys(map);
    const list = await Promise.all(keys.map(key => fn(map[key])));
    return keys.reduce((acc, key, index) => _objectSpread({}, acc, {
      [key]: list[index]
    }), {});
  }

  constructor() {
    this.addResolver = (branch, resolver, refs = {}) => {
      this.branches[branch] = {
        resolver,
        refs: Object.keys(refs).reduce((acc, key) => {
          const refConfig = typeof refs[key] === 'string' ? {
            branch: refs[key]
          } : refs[key];

          if (!refConfig.foreignProperty) {
            refConfig.foreignProperty = '_id';
          }

          return _objectSpread({}, acc, {
            [key]: refConfig
          });
        }, {}),
        data: null
      };
    };

    this.load = data => {
      this.data = data;
      return Promise.all(Object.keys(this.data).map(this.seedBranch.bind(this)));
    };

    this.branches = {};
  }

  async seedBranch(branch) {
    if (!this.branches[branch].promise) {
      this.branches[branch].promise = new Promise(async resolve => {
        const dataWithReferences = await this.linkReferences(branch);
        this.branches[branch].data = await this.constructor.resolveMap(dataWithReferences, this.branches[branch].resolver);
        resolve();
      });
    }

    return this.branches[branch].promise;
  }

  async linkReferences(branch) {
    const {
      refs
    } = this.branches[branch];
    await Promise.all(Object.values(refs).map(ref => this.seedBranch(ref.branch)));
    const items = this.data[branch];
    return Object.keys(items).reduce((acc, index) => {
      const itemData = items[index];
      const item = (0, _omit.default)(itemData, Object.keys(refs));
      Object.keys(refs).forEach(key => {
        const ref = refs[key];

        if (!(0, _has.default)(itemData, key)) {
          return;
        }

        const itemRefValue = (0, _get.default)(itemData, key);

        if (Array.isArray(itemRefValue)) {
          (0, _set.default)(item, ref.localProperty || `${key}Ids`, itemRefValue.map(alias => this.branches[ref.branch].data[alias][ref.foreignProperty]));
        } else {
          (0, _set.default)(item, ref.localProperty || `${key}Id`, this.branches[ref.branch].data[itemRefValue][ref.foreignProperty]);
        }
      });
      return _objectSpread({}, acc, {
        [index]: item
      });
    }, {});
  }

}

var _default = Seeder;
exports.default = _default;