"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _mongodb = require("mongodb");

var _invariant = _interopRequireDefault(require("invariant"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

class Store {
  constructor(collection) {
    this._collection = collection;
    this.id = '_id';
  }

  get collection() {
    (0, _invariant.default)(this._collection, `${this.constructor.name} store missing collection!`);
    return this._collection;
  }

  set collection(collection) {
    this._collection = collection;
  }

  hasCollection() {
    return this._collection !== undefined;
  } // eslint-disable-next-line class-methods-use-this


  parse(data) {
    return data;
  }

  query(fn) {
    (0, _invariant.default)(fn && typeof fn === 'function', 'Requires a function that will receive the collection!');
    return fn(this.collection);
  }

  async aggregate({
    pipeline,
    options
  } = {}) {
    (0, _invariant.default)(pipeline, 'Pipeline is required!');
    return this.collection.aggregate(pipeline, options);
  }

  findById(id) {
    (0, _invariant.default)(id, 'Id is required!');
    return this.collection.findOne({
      [this.id]: this.toId(id)
    });
  }

  findOne({
    query = {},
    options = {}
  } = {}) {
    return this.collection.findOne(query, options);
  }

  async findMany(_ref = {}) {
    let {
      query = {},
      orderBy,
      limit,
      skip,
      fields
    } = _ref,
        options = _objectWithoutProperties(_ref, ["query", "orderBy", "limit", "skip", "fields"]);

    if (fields) {
      if (Array.isArray(fields)) {
        Object.assign(options, {
          projection: fields.reduce((acc, field) => _objectSpread({}, acc, {
            [field]: 1
          }), {})
        });
      } else {
        Object.assign(options, {
          projection: fields
        });
      }
    }

    let cursor = await this.collection.find(query, options);

    if (orderBy) {
      cursor = cursor.sort(Object.keys(orderBy).reduce((acc, field) => _objectSpread({}, acc, {
        [field]: orderBy[field] === 1 || typeof orderBy[field] === 'string' && orderBy[field].toLowerCase() === 'asc' ? 1 : -1
      }), {}));
    }

    if (skip) {
      cursor = cursor.skip(skip);
    }

    if (limit) {
      cursor = cursor.limit(limit);
    }

    return options.cursor ? cursor : cursor.toArray();
  }

  async findOneOrCreate(data) {
    return (await this.findOne({
      query: data
    })) || this.createOne(data);
  }

  async createOne(data, options) {
    (0, _invariant.default)(typeof data === 'object', 'Data is required!');
    const result = await this.collection.insertOne((await this.parse(data)), options);
    return result.ops[0];
  }

  async createMany(data, options) {
    (0, _invariant.default)(data && Array.isArray(data), 'Requires an array of items!');
    const result = await this.collection.insertMany((await Promise.all(data.map(this.parse.bind(this)))), options);
    return result.ops;
  }

  async updateOne({
    query = {},
    update,
    options = {
      returnOriginal: false
    }
  } = {}) {
    (0, _invariant.default)(typeof update === 'object', 'Update payload is required!');
    const result = await this.collection.findOneAndUpdate(query, update, options);
    return result.value;
  }

  async updateMany({
    query = {},
    update,
    options = {}
  } = {}) {
    (0, _invariant.default)(typeof update === 'object', 'Update payload is required!');
    return this.collection.updateMany(query, update, options);
  }

  async replaceOne({
    query,
    replacement,
    options
  }) {
    (0, _invariant.default)(query, 'Query is required!');
    (0, _invariant.default)(replacement, 'replacement is required!');
    const result = await this.collection.findOneAndReplace(query, (await this.parse(replacement)), options);
    return result.value;
  }

  async save(data) {
    return data[this.id] ? this.replaceOne({
      query: {
        [this.id]: this.toId(data[this.id])
      },
      replacement: data
    }) : this.createOne(data);
  }

  async deleteOne({
    query = {},
    options = {}
  } = {}) {
    const result = await this.collection.findOneAndDelete(query, options);
    return result.value;
  }

  async deleteMany({
    query = {},
    options = {}
  } = {}) {
    return this.collection.deleteMany(query, options);
  }

  count({
    query = {},
    options = {}
  } = {}) {
    return this.collection.countDocuments(query, options);
  }

  toId(id) {
    return this.constructor.toId(id);
  }

}

Store.toId = id => (0, _mongodb.ObjectID)(id);

var _default = Store;
exports.default = _default;