"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getTimestampFromMigrationFile = exports.queryToPipeline = exports.buildSort = exports.parseMongoQuery = exports.toMongo = exports.fromMongoArray = exports.fromMongoCursor = exports.fromMongo = exports.idPattern = exports.idToQuery = exports.toBSON = void 0;

var _mongodb = require("mongodb");

var _mongodbExtendedJson = _interopRequireDefault(require("mongodb-extended-json"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const toBSON = query => _mongodbExtendedJson.default.parse(typeof query !== 'string' ? JSON.stringify(query) : query);

exports.toBSON = toBSON;

const idToQuery = id => ({
  _id: (0, _mongodb.ObjectID)(id)
});

exports.idToQuery = idToQuery;
const idPattern = /^(?=[a-f\d]{24}$)(\d+[a-f]|[a-f]+\d)/i;
exports.idPattern = idPattern;

const fromMongo = item => _objectSpread({}, item, {
  id: item._id.toString()
});

exports.fromMongo = fromMongo;

const fromMongoCursor = data => data.map(fromMongo).toArray();

exports.fromMongoCursor = fromMongoCursor;

const fromMongoArray = data => data.map(fromMongo);

exports.fromMongoArray = fromMongoArray;

const toMongo = item => _objectSpread({}, item, {
  _id: (0, _mongodb.ObjectID)(item.id)
});

exports.toMongo = toMongo;

const parseMongoQuery = params => {
  const query = {};

  if (params.orderBy) {
    query.orderBy = params.orderBy;
  }

  if (params.filters) {
    query.query = toBSON(params.query);
  }

  if (params.offset !== undefined) {
    query.skip = parseInt(params.offset, 10);
  }

  if (params.limit !== undefined) {
    query.limit = parseInt(params.limit, 10);
  }

  return query;
};

exports.parseMongoQuery = parseMongoQuery;

const buildSort = orderBy => Object.keys(orderBy).reduce((acc, field) => _objectSpread({}, acc, {
  [field]: orderBy[field] === 1 || typeof orderBy[field] === 'string' && orderBy[field].toLowerCase() === 'asc' ? 1 : -1
}), {});

exports.buildSort = buildSort;

const queryToPipeline = ({
  query,
  orderBy,
  skip,
  limit
}, pipeline = []) => {
  if (query) {
    pipeline.push({
      $match: query
    });
  }

  if (orderBy) {
    pipeline.push({
      $sort: buildSort(orderBy)
    });
  }

  if (skip) {
    pipeline.push({
      $skip: skip
    });
  }

  if (limit) {
    pipeline.push({
      $limit: limit
    });
  }

  return pipeline;
};

exports.queryToPipeline = queryToPipeline;

const getTimestampFromMigrationFile = file => {
  const fileName = file.substr(file.lastIndexOf('/') + 1);
  return fileName.substr(0, fileName.indexOf('_'));
};

exports.getTimestampFromMigrationFile = getTimestampFromMigrationFile;